#include "main.h"
#include "globals.h"
#include "nkeys.h"
#include "joy.h"
#include "gpadpro.h"
#include "sidewind.h"
#include "psxpad.h"
#include "control.h"

static int mleft[2];
static int mright[2];
static int mup[2];
static int mdown[2];
static int centx[2];
static int centy[2];

extern BYTE lowkeys[NO_OF_KEYS+1];
static BYTE joykeys[NO_OF_KEYS+1];
extern pINPUTS pInputs;
static int NoJoys;
static int NoSidewinders;
static int NoPSX;
static int NoGamePadPros;

struct sSidewinder SideWinder;
struct sPSXPAD PSXPAD;

//default joystick setup
static int sdefjoyidx[NO_JOY_CONFIGS]=
{
   K_PL1_B1,
   K_PL1_B2,
   K_PL1_B3,
   K_UI,
   K_UI_SELECT,
   K_PAUSE,
   K_RESET,
   K_ESCAPE,
   K_COIN1,
   K_PL1_START,
   K_PL1_B1,
   K_PL1_B2,
   K_PL1_B3,
   K_THROTTLE,
   K_SCREENSHOT,
   K_FPS,
   K_RESET,
   K_ESCAPE,
   K_COIN2,
   K_PL2_START
};

int joyidx[NO_JOY_CONFIGS];
int defjoyidx[NO_JOY_CONFIGS];


void checkcontroller(void)
{
   memset(joykeys,0,NO_OF_KEYS+1);
   switch(joytype)
   {
      case JOY_2BUTTON:
      case JOY_4BUTTON:
      case JOY_6BUTTON:
      case JOY_8BUTTON:
         NoJoys=checkjoy();
         if(NoJoys<1)
            joytype=0;
         else
            calibratejoy();

         break;
      case JOY_SIDEWIND:
         NoSidewinders=GetSidewinderStatus(&SideWinder);
         if(NoSidewinders<1)
            joytype=0;
         break;
      case JOY_GAMEPADPRO:
         NoGamePadPros=checkgpp();
         if(NoGamePadPros<1)
            joytype=0;
         break;
      case JOY_PSXPAD:
         NoPSX=psx1_init();
         if(NoPSX<1)
            joytype=0;

   }
}


void handlejoystick(void)
{
   switch(joytype)
   {
      case JOY_2BUTTON:
      case JOY_4BUTTON:
      case JOY_6BUTTON:
      case JOY_8BUTTON:
         readstandardjoy();
         break;
      case JOY_SIDEWIND:
         readsidewinder();
         break;
      case JOY_GAMEPADPRO:
         readgamepadpro();
         break;
      case JOY_PSXPAD:
         readpsxpad();
	}
}

void calibratejoy(void)
{
   int dummy,nojoys,i;
   int minx,miny,maxx,maxy;
   int workx,worky,workx2,worky2;

   centx[0]=centy[0]=centx[1]=centy[1]=0;
   if(joytype==JOY_6BUTTON||joytype==JOY_8BUTTON)
      nojoys=2;
   else
      nojoys=NoJoys;

//get centre position
   for(i=0;i<8;i++)
   {
      readjoystick(nojoys,&dummy,&workx,&worky,&workx2,&worky2);
      centx[0]+=workx;
      centy[0]+=worky;
      centx[1]+=workx2;
      centy[1]+=worky2;
   }

   centx[0]>>=3;
   centy[0]>>=3;
   centx[1]>>=3;
   centy[1]>>=3;


   for(i=0;i<nojoys;i++)
   {
//make up min max values
      minx=centx[i]-(centx[i]>>1);
      miny=centy[i]-(centy[i]>>1);
      maxx=centx[i]+(centx[i]>>1);
      maxy=centy[i]+(centy[i]>>1);
//make up some believable deltas
      mleft[i]=minx+((centx[i]-minx)/8);
      mright[i]=maxx-((maxx-centx[i])/8);
      mup[i]=miny+((centy[i]-miny)/8);
      mdown[i]=maxy-((maxy-centy[i])/8);
   }
}


void readstandardjoy(void)
{
   int i,joybuttons,n,nbut,p2,bmask;
   int injoyx[2];
   int injoyy[2];
   int nojoys=NoJoys;

   readjoystick(NoJoys,&joybuttons,&injoyx[0],&injoyy[0],&injoyx[1],&injoyy[1]);

   bmask=16;
   if(joytype==JOY_2BUTTON)
   {
//2 possible joysticks
      for(i=0;i<NoJoys;i++)
      {
         p2=i*10;
         if(!(joybuttons&bmask))
            JOYSET(p2)
         else
            JOYCLEAR(p2)

         bmask<<=1;

         if(!(joybuttons&bmask))
            JOYSET(1+p2)
         else
            JOYCLEAR(1+p2)
         bmask<<=1;
      }
   }
   else
   {
//too many buttons - so there can only be one joystick
      for(n=0;n<4;n++)
      {
         if(!(joybuttons&bmask))
            JOYSET(n)
         else
            JOYCLEAR(n)
         bmask<<=1;
      }
   }

//six button, 2 buttons are second x/y axis
   if(joytype==JOY_6BUTTON)
   {

      if(injoyx[1]==(centx[1]-128))
         JOYSET(4)
      else
         JOYCLEAR(4)

      if(injoyy[1]==(centy[1]-128))
         JOYSET(5)
      else
         JOYCLEAR(5)
   }
//8button, 4 buttons mapped to second x/y axis
   if(joytype==JOY_8BUTTON)
   {
      if(injoyx[1]<mleft[1])
         JOYSET(4)
      else
         JOYCLEAR(4)

      if(injoyy[1]<mup[1])
         JOYSET(5)
      else
         JOYCLEAR(5)

      if(injoyx[1]>mright[1])
         JOYSET(6)
      else
         JOYCLEAR(6)

      if(injoyy[1]>mdown[1])
         JOYSET(7)
      else
         JOYCLEAR(7)

   }

//copy across joystick movement
   for(i=0;i<NoJoys;i++)
   {
      p2=i<<3;
      if(injoyy[i]>mdown[i])
      {
         JOYDIRSET(K_PL1_DOWN+p2)
         JOYDIRSET(K_UI_DOWN)
      }
      else
      {
         JOYDIRCLEAR(K_PL1_DOWN+p2)
         JOYDIRCLEAR(K_UI_DOWN)
      }

      if (injoyy[i]<mup[i])
      {
         JOYDIRSET(K_PL1_UP+p2)
         JOYDIRSET(K_UI_UP)
      }
      else
      {
         JOYDIRCLEAR(K_PL1_UP+p2)
         JOYDIRCLEAR(K_UI_UP)
      }

      if (injoyx[i]<mleft[i])
      {
         JOYDIRSET(K_PL1_LEFT+p2)
         JOYDIRSET(K_UI_LEFT)
      }
      else
      {
         JOYDIRCLEAR(K_PL1_LEFT+p2)
         JOYDIRCLEAR(K_UI_LEFT)
      }

      if (injoyx[i]>mright[i])
      {
         JOYDIRSET(K_PL1_RIGHT+p2)
         JOYDIRSET(K_UI_RIGHT)
      }
      else
      {
         JOYDIRCLEAR(K_PL1_RIGHT+p2)
         JOYDIRCLEAR(K_UI_RIGHT)
      }
   }
}

void readsidewinder(void)
{
   int i,nojoys,p2;
   static unsigned long int swdidx[4]={SW_UP,SW_DOWN,SW_LEFT,SW_RIGHT};
   static unsigned long int swbidx[10]={SW_A,SW_B,SW_C,SW_X,SW_Y,SW_Z,SW_LEFTFINGER,SW_RIGHTFINGER,SW_M,SW_START};

//get the details
   nojoys=GetSidewinderStatus(&SideWinder);
   if(nojoys<1)
      return;

   if(nojoys>2)
      nojoys=2;

   for(i=0;i<nojoys;i++)
      setgenjoy(SideWinder.dwSidewinder[i],i,swdidx,swbidx);
}


void readgamepadpro(void)
{
   int i,nojoys,bidx;
   unsigned long int inval;
   static unsigned long int gpdidx[4]={GPP_UP,GPP_DOWN,GPP_LEFT,GPP_RIGHT};
   static unsigned long int gpbidx[10]={GPP_RED,GPP_YELLOW,GPP_GREEN,GPP_BLUE,GPP_L1,GPP_R1,GPP_L2,GPP_R2,GPP_SELECT,GPP_START};

   nojoys=NoGamePadPros;

   for(i=0;i<nojoys;i++)
   {
      inval=readgpp(i);
      if(inval==1)
         return;
      setgenjoy(inval,i,gpdidx,gpbidx);
   }

}

void readpsxpad(void)
{
   int i,nojoys,p2,bidx;
   static unsigned long int pdidx[]={PSX_UP,PSX_DOWN,PSX_LEFT,PSX_RIGHT};
   static unsigned long int pbidx[]=
      {PSX_CROSS,PSX_CIRCLE,PSX_TRIANGLE,PSX_SQUARE,PSX_L1,PSX_R1,PSX_L2,PSX_R2,PSX_SELECT,PSX_START};

//get the details
   nojoys=psx1_poll(&PSXPAD);
   if(nojoys<1)
      return;

   if(nojoys>2)
      nojoys=2;
   for(i=0;i<nojoys;i++)
      setgenjoy(PSXPAD.dwPSXPAD[i],i,pdidx,pbidx);

}

void setgenjoy(unsigned long int inval,int joyno,unsigned long int *dirs,unsigned long int *buts)
{
   int i;
   int p2=(joyno*10);
//directions
   for(i=0;i<4;i++)
   {
      if(inval&dirs[i])
      {
         JOYDIRSET(i)
         JOYDIRSET(K_UI_UP+i)
      }
      else
      {
         JOYDIRCLEAR(i)
         JOYDIRCLEAR(K_UI_UP+i)
      }
   }
//buttons
   for(i=0;i<10;i++)
   {
      if(inval&buts[i])
         JOYSET(i+p2)
      else
         JOYCLEAR(i+p2)
   }

}

void getdefjoy(void)
{
   int n;
   FILE *fJoy;
   static char buffer[256];
//set to default
   for(n=0;n<NO_JOY_CONFIGS;n++)
   {
      joyidx[n]=sdefjoyidx[n];
      defjoyidx[n]=sdefjoyidx[n];
   }
//try to open up default keyboard file
   fJoy=fopen("keys\\default.joy","rb");
   if(fJoy)
   {
      fread(joyidx,sizeof(int),NO_JOY_CONFIGS,fJoy);
      memcpy(defjoyidx,joyidx,NO_JOY_CONFIGS*sizeof(int));
      fclose(fJoy);
   }
//then get keys for this game
   sprintf(buffer,"keys\\%s.joy",pBoard->name);
   fJoy=fopen(buffer,"rb");
   if(fJoy)
   {
      GameHasSetJoy=TRUE;
      fread(joyidx,sizeof(int),NO_JOY_CONFIGS,fJoy);
      fclose(fJoy);
   }
}


//save default joy settings
void savedefjoy(void)
{
   FILE *fDefJoy;
   fDefJoy=fopen("keys\\default.joy","wb");
   if(fDefJoy)
   {
      fwrite(defjoyidx,sizeof(int),NO_JOY_CONFIGS,fDefJoy);
      fclose(fDefJoy);
   }
}

//save joy settings for this game
void savegamejoy(void)
{
   FILE *fGameJoy;
   char buffer[256];

   if(!GameHasSetJoy)
      return;
   sprintf(buffer,"keys\\%s.joy",pBoard->name);
   fGameJoy=fopen(buffer,"wb");
   if(fGameJoy)
   {
      fwrite(joyidx,sizeof(int),NO_JOY_CONFIGS,fGameJoy);
      fclose(fGameJoy);
   }
}
