#include "main.h"
#include "galsnd.h"
#include "globals.h"

static int lfofreq=0;
static int lfobit[4];
static pBYTE pGalOut;
static int GalStep;
static int GalShift;
static int updpos;
static int updlast;
static int updlen;
static int zigzaglatch;

static GALSAMPSHIFT GalSampShift[3]={{64,6},{32,5},{16,4}};

static BYTE tonewave[4][TOOTHSAW_LENGTH];
static int pitch,vol;


static struct AY8910interface interface =
{
   1,
   1789750,
   {15},
   {2},
   {0},
   {0},
   {0},
   {0}
};


void galreset(void)
{
   lfofreq=0;
   memset(lfobit,0,4*sizeof(int));
}



void lfoon(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   value&=1;
   if(value&&!channel[2].playing)
      playsample(2,2,LOOP,BACK_FREQ);
   if(!value)
      stopsample(2);
}



void backlfo(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   int work;


   work=offset-mwb->lowAddr;
   value&=1;
   if(lfobit[work]==value)
      return;

   lfobit[work]=value;

   if(value)
      lfofreq|=(1<<work);
   else
      lfofreq&=~(1<<work);

//just to give us a bit of movement in the background noise - not accurate
   if(channel[2].sampleno==2)
      setfreq(2,BACK_FREQ);


}



void shootw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   static int lastval;
   value&=1;
   if(value&&!lastval)
      playsample(0,0,ONE_SHOT,DEFAULT_FREQ);
   lastval=value;
}

void deathw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   static int lastval;
   value&=1;
   if(value&&!lastval)
      playsample(0,1,ONE_SHOT,DEFAULT_FREQ);
   lastval=value;

}



static void tone_update(pBYTE buffer, int length)
{
	int i,j;
   BYTE *w = tonewave[vol];
	static int counter, countdown;

   if(!length)
      return;

   buffer+=updpos;
   if( pitch != (0xff) )
	{
      i=length;
      while(i)
		{
			int mix = 0;

         for (j = 0;j < GalStep;j++)
			{
            if (countdown>=256)
				{
               counter++;
               counter&=15;
               countdown = pitch;
				}
				countdown++;
				mix += w[counter];
			}
         *buffer++ = (mix>>GalShift)^0x80;
         i--;
		}
	}
	else
	{
      for(i=length;i;i-- )
			*buffer++ = 0;
	}
}

void galpitchw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   galpartupdate();
   pitch=value;
}

void galvolw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   int work=mwb->lowAddr-offset;
   galpartupdate();
   vol=(vol&~(1<<work))|((value&1)<<work);
}





void galsndinit(void)
{
	int i, j, sweep, charge, countdown, generator, bit1, bit2;


	memset(tonewave, 0, sizeof(tonewave));
   pGalOut=myalloc(sizeof(BYTE)*actsamples);

   GalStep=GalSampShift[sampleidx].step;
   GalShift=GalSampShift[sampleidx].shift;

	for( i = 0; i < TOOTHSAW_LENGTH; i++ )
	{
		#define V(r0,r1) 2*TOOTHSAW_AMPLITUDE*(r0)/(r0+r1)-TOOTHSAW_AMPLITUDE
		double r0a = 1.0/1e12, r1a = 1.0/1e12;
		double r0b = 1.0/1e12, r1b = 1.0/1e12;

		/* #0: VOL1=0 and VOL2=0
		 * only the 33k and the 22k resistors R51 and R50
		 */
		if( i & 1 )
		{
			r1a += 1.0/33000;
			r1b += 1.0/33000;
		}
		else
		{
			r0a += 1.0/33000;
			r0b += 1.0/33000;
		}
		if( i & 4 )
		{
			r1a += 1.0/22000;
			r1b += 1.0/22000;
		}
		else
		{
			r0a += 1.0/22000;
			r0b += 1.0/22000;
		}
		tonewave[0][i] = V(1.0/r0a, 1.0/r1a);

		/* #1: VOL1=1 and VOL2=0
		 * add the 10k resistor R49 for bit QC
		 */
		if( i & 4 )
			r1a += 1.0/10000;
		else
			r0a += 1.0/10000;
		tonewave[1][i] = V(1.0/r0a, 1.0/r1a);

		/* #2: VOL1=0 and VOL2=1
		 * add the 15k resistor R52 for bit QD
		 */
		if( i & 8 )
			r1b += 1.0/15000;
		else
			r0b += 1.0/15000;
		tonewave[2][i] = V(1.0/r0b, 1.0/r1b);

		/* #3: VOL1=1 and VOL2=1
		 * add the 10k resistor R49 for QC
		 */
		if( i & 4 )
			r0b += 1.0/10000;
		else
			r1b += 1.0/10000;
		tonewave[3][i] = V(1.0/r0b, 1.0/r1b);
	}

	pitch = 0;
	vol = 0;
   updlen=updpos=updlast=0;
}



void galsndupd(void)
{
   int i;
   tone_update(pGalOut,actsamples-updpos);
   playstreamedsample(0,pGalOut,actsamples,10);
   updpos=updlast=0;
}

void galpartupdate(void)
{

   int work;
//get ticks this frame
   work=SAMPLESSOFAR;

//update since last
   updlen=work-updlast;
   tone_update(pGalOut,updlen);
//update position
   updpos+=updlen;
//store this tick
   updlast=work;

}

void initzigzagsnd(void)
{
   AY8910_sh_start(&interface);
}


void zigzaglatchw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   zigzaglatch=offset-mwb->lowAddr;
}

void zigzagdataw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   struct z80PortWrite zpw;
   AY8910_write_port_0_w(0,zigzaglatch,&zpw);
}

void zigzagcontrolw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   struct z80PortWrite zpw;
   AY8910_control_port_0_w(0,zigzaglatch,&zpw);
}

void checkmansndcmd(UINT16 offset, UINT8 value, struct z80PortWrite *zpw)
{
   struct MemoryWriteByte mwb;
   sound_command_w(0,value,&mwb);
   causenmi(1);
}

