#include "main.h"
#include "globals.h"

//Sound Blaster routines
int actchannels;
unsigned int sbbase;
unsigned int sbint;
unsigned int sbdma;
unsigned int sbvect;
int DSPVersion;
signed short volume[NO_VOL_STEPS][256];
unsigned short tempmixbuf[512];
volatile int busy=0;
volatile int inirq=0;

unsigned char stackbucket[8192];
unsigned char *istack;

void (*sbpoint) (void);


unsigned int  CBuffer;
int           DOSBufOfs;

_go32_dpmi_seginfo oldVect;
_go32_dpmi_seginfo newVect;
extern int vol;

int checksb(void)
{
   char *sb;
//get SB enviroment variable
   sb=getenv("BLASTER");
   if(!sb)
   {
      printf("BLASTER environment variable not defined\nSound card cannot be intiailised\n");
      return FALSE;
   }
//get the relevant details
   getsbval(sb,"A",&sbbase);
   getsbval(sb,"I",&sbint);
   getsbval(sb,"D",&sbdma);
//attemp to reset the DSP
   if(!resetDSP())
   {
      printf("Unable to intialise Sound Card,\ncheck BLASTER environment variable is set correctly\n");
      return FALSE;
   }
   sbpoint=sbIRQ;
   return TRUE;
}

int initsb(void)
{
   BYTE pic;
//setup stack
   istack=stackbucket+8160;

//lock all the IRQ stuff in memory
   _go32_dpmi_lock_data(&syspaused,sizeof(int));
   _go32_dpmi_lock_data(&sbbase,sizeof(int));
   _go32_dpmi_lock_data(&sbint,sizeof(int));
   _go32_dpmi_lock_data(&sbdma,sizeof(int));

   _go32_dpmi_lock_data(tempmixbuf,sizeof(short)*512);
   _go32_dpmi_lock_data(volume,sizeof(short)*NO_VOL_STEPS*256);
   _go32_dpmi_lock_data(&CBuffer,sizeof(int));
   _go32_dpmi_lock_data(&DOSBufOfs,sizeof(int));
   _go32_dpmi_lock_data(&DosDataSeg,sizeof(short));
   _go32_dpmi_lock_data(&actchannels,sizeof(int));
   _go32_dpmi_lock_data((void *)&busy,sizeof(int));
   _go32_dpmi_lock_data((void *)&inirq,sizeof(int));
   _go32_dpmi_lock_data(&istack,sizeof(unsigned char *));
   _go32_dpmi_lock_data(stackbucket, 8192);
   _go32_dpmi_lock_data(&sbpoint,sizeof(long));
   _go32_dpmi_lock_code((void *)sbIRQ,(long)endofIRQ-(long)sbIRQ);
   _go32_dpmi_lock_code((void *)setfreq,(long)getsbval-(long)setfreq);
   _go32_dpmi_lock_code((void *)playsample,(long)playsamplepos-(long)playsample);
   _go32_dpmi_lock_code((void *)playsamplepos,(long)sbIRQ-(long)playsamplepos);
   _go32_dpmi_lock_code((void *)_sbwrap,(long)_sbwrap_end-(long)_sbwrap);
   _go32_dpmi_lock_data(&DSPVersion,sizeof(int));


   actchannels=pBoard->channels;
//look for a SB (enviroment var check + attempt to initialise DSP)
   if(!checksb())
      return FALSE;
//get the DSP version
   DSPVersion=getDSPVersion();
//we dont handle very old cards without auto DMA buffer swapping
   if(DSPVersion<=0x200)
   {
      printf("Sound Card does not have auto DMA playback\nUnable to initialise Sound Card");
      return FALSE;
   }
//turn off stereo if we need to
   if(DSPVersion>=0x300&&DSPVersion<0x400)
      disablestereo();
   createvoltable();

//DMA buffer
   if(!allocDMA())
      return FALSE;
//IRQ service routine
   CBuffer=1;


   pic=inportb(0x21);

   if(sbint>7)
   {
      outportb(0x21,pic&0xfb);
      pic=inportb(0xa1);
      outportb(0xa1,pic&(~(1<<(sbint-8))));
   }
   else
      outportb(0x21,pic&(~(1<<sbint)));


   setsbvect(sbint);
//set DMA going
   startplayback();
   return TRUE;
}

int createvoltable(void)
{
   int i,j;
//volume lookup table
   if(pBoard->voladjust)
   {
      for(j=0;j<NO_VOL_STEPS;j++)
      {
         for(i=0;i<256;i++)
         {
            volume[j][i]=(i-128)*j*256/NO_VOL_STEPS/pBoard->voladjust;
            volume[j][i]=(signed short)(((double)volume[j][i]/15.0F)*(double)vol);
         }
      }
   }
   else
   {
      for(j=0;j<NO_VOL_STEPS;j++)
      {
         for(i=0;i<256;i++)
         {
            volume[j][i]=(i-128)*j*256/NO_VOL_STEPS/actchannels;
            volume[j][i]=(signed short)(((double)volume[j][i]/15.0F)*(double)vol);
         }
      }
   }
   return 0;
}

int resetDSP(void)
{
   int i;
   int port=sbbase+6;

   outportb(port,1);
   for(i=0;i<8;i++)
      inportb(port);
   outportb(port,0);

   if(readDSP()!=0xaa)
      return FALSE;

   return TRUE;
}

void writeDSP(BYTE val)
{
  while((inportb(sbbase+0x0c)&0x80)==0x80);
  outportb(sbbase+0x0c,val);
}

int readDSP(void)
{
   int i;
   for(i=0;i<0xffff;i++)
   {
      if(inportb(sbbase+0x0e)&0x80)
         return inportb(sbbase+0x0a);
   }
   return -1;
}

int getDSPVersion(void)
{
   int ret,t1,t2;

   writeDSP(0xe1);

   t1=readDSP();
   t2=readDSP();

   ret=(t1<<8)|t2;

   return ret;
}

void clearsb(void)
{
   int i;

   BYTE pic;
   writeDSP(0xd0);
   writeDSP(0xda);

  //Mask IRQ
   if(sbint>7)
   {
      pic=inportb(0xa1);
      outportb(0xa1,pic&(1<<(sbint-8)));
   }
   else
   {
      pic=inportb(0x21);
      outportb(0x21,pic&(1<<sbint));
   }


   unlockdpmidata(channel,sizeof(CHANNEL)*NO_CHANNELS);
   unlockdpmidata(&sbbase,sizeof(int));
   unlockdpmidata(&sbint,sizeof(int));
   unlockdpmidata(&sbdma,sizeof(int));
   unlockdpmidata(&syspaused,sizeof(int));
   unlockdpmidata(tempmixbuf,sizeof(short)*512);
   unlockdpmidata(volume,sizeof(short)*NO_VOL_STEPS*256);
   unlockdpmidata(&CBuffer,sizeof(int));
   unlockdpmidata(&DOSBufOfs,sizeof(int));
   unlockdpmidata(&DosDataSeg,sizeof(short));

   _go32_dpmi_set_protected_mode_interrupt_vector(sbvect,&oldVect);
}


int allocDMA(void)
{
   unsigned long page1,page2;
   int seg,sel,outaddr,i;
//1K needed, allocate 2K, just incase we cross a page boundary
   seg=__dpmi_allocate_dos_memory((2048+15)>>4,&sel);

   if(seg<0)
   {
      printf("Unable to allocate DMA buffer\n");
      return FALSE;
   }

   DOSBufOfs=seg<<4;
//check for a page crossing
   page1=DOSBufOfs>>16;
   page2=(DOSBufOfs+1024)>>16;
//use second half of 2K
   if(page1!=page2)
      DOSBufOfs+=1024;
//Clear DMA buffer
   outaddr=DOSBufOfs;
   _farsetsel(DosDataSeg);
   for(i=0;i<0x100;i++)
      _farnspokel(outaddr+=4,0x80808080);
   return TRUE;
}


void startplayback(void)
{
   unsigned int  Page,OffSet;

   writeDSP(0xd1);
   writeDSP(0x40);
   writeDSP(210);


   Page=DOSBufOfs>>16;
   OffSet=DOSBufOfs&0xffff;
   outportb(0x0a,4|sbdma);
   outportb(0x0c,0);
   outportb(0x0b,0x58|sbdma);
   outportb(sbdma<<1,OffSet&0xff);
   outportb(sbdma<<1,OffSet>>8);

  switch(sbdma)
  {
      case 0:
         outportb(0x87,Page);
         break;
      case 1:
         outportb(0x83,Page);
         break;
      case 3:
         outportb(0x82,Page);
         break;
  }


  outportb((sbdma<<1)+1,0xff);
  outportb((sbdma<<1)+1,0x03);

  outportb(0x0a,sbdma);
  writeDSP(0x48);
  writeDSP(0xff);
  writeDSP(0x01);


  writeDSP(0x1c);

}

void initDMA(void)
{
   CBuffer=((0x03ff-(inportb((sbdma<<1)+1)|(inportb((sbdma<<1)+1))<<8))>>9)+1;
}




void playsample(int cno,int sampleno,int looping,int freq)
{

   static int first=TRUE;

   if(first)
   {
      CBuffer=((0x03ff-(inportb((sbdma<<1)+1)|(inportb((sbdma<<1)+1))<<8))>>9)+1;
      first=FALSE;
   }

   playsamplepos(cno,sampleno,looping,freq,0);
}

void playsamplepos(int cno,int sampleno,int looping,int freq,fixed pos)
{
   if(sampleno<NO_SAMPLES&&cno<NO_CHANNELS)
   {
      if(sampledata[sampleno].loaded)
      {
         channel[cno].playing=1;
         channel[cno].sampleno=sampleno;
         channel[cno].looping=looping;
         channel[cno].pSample=&sampledata[sampleno];
         channel[cno].samplepos=pos;
         channel[cno].samplelen=int2fixed(sampledata[sampleno].length);
         setfreq(cno,freq);
         channel[cno].queueno=-1;
      }
   }
}


static void sbIRQ(void)
{
   int cno;
   int samp;
   int actbuf;
   long *ptemp;
   int i;
   BYTE isr;
   pSAMPLE pSample;
   int outaddr;
   unsigned short *lkup;


   if(DSPVersion>=0x400)
   {
      outportb(sbbase+4,0x82);
      isr=inportb(sbbase+5)&7;

      if(isr&4||(!(isr&3)))
      {
         SBINT_ACK
         return;
      }
   }



   CBuffer++;
   CBuffer&=1;


   ptemp=(long *)tempmixbuf;

   if(busy)
   {
      SBINT_ACK
      return;
   }


   if(syspaused&&!busy)
   {
      busy=1;
      outaddr=DOSBufOfs+(CBuffer<<9);
      for(i=0;i<256;i++)
         ptemp[i]=0x80808080;

      _farsetsel(DosDataSeg);
      for(samp=0;samp<512;samp++)
         _farnspokeb(outaddr++,tempmixbuf[samp]>>8);

      SBINT_ACK
      busy=0;
      return;
   }



   if(!busy&&!syspaused)
   {

      outaddr=DOSBufOfs+(CBuffer<<9);

      for(i=0;i<256;i++)
         ptemp[i]=0x80808080;


      asm volatile ("sti");
      for(cno=0;cno<actchannels;cno++)
      {
         if(channel[cno].playing&&channel[cno].vol)
         {
            lkup=(unsigned short int *)(volume[channel[cno].vol]);
            pSample=channel[cno].pSample;

            for(samp=0;samp<512;samp++)
            {
               tempmixbuf[samp]+=lkup[pSample->sample[fixed2int(channel[cno].samplepos)]];
               channel[cno].samplepos+=channel[cno].sampledelta;
               if(channel[cno].samplepos>channel[cno].samplelen)
               {
                  if(channel[cno].looping&&channel[cno].looping!=LOOP)
                     channel[cno].looping--;

                  channel[cno].playing=channel[cno].looping;
                  channel[cno].samplepos=FIXED_ZERO;
                  if(!channel[cno].playing)
                  {
                     if(channel[cno].queueno>=0)
                     {
                        playsamplepos(cno,channel[cno].queueno,channel[cno].queueloop,
                                       channel[cno].queuefreq,channel[cno].queuepos);
                        pSample=channel[cno].pSample;
                        channel[cno].queueno=-1;
                     }
                     else
                        break;
                  }
               }
            }
         }
      }

      _farsetsel(DosDataSeg);
      for(samp=0;samp<512;samp++)
         _farnspokeb(outaddr++,tempmixbuf[samp]>>8);

      asm volatile ("cli");
   }

   SBINT_ACK

   busy=0;
   return;
}

static void endofIRQ(void)
{
   return;
}
void setfreq(int cno,int freq)
{
   double outfreq;

   if(cno<NO_CHANNELS&&freq>0)
   {
      outfreq=(double)(1.0f/(1000.0f/(double)freq));
      channel[cno].sampledelta=float2fixed(outfreq);
   }
}

void getsbval(char *sb,char *ident,int *outval)
{
   static char bucket[256];
   char *pWork;
   int i,p;

   pWork=strstr(sb,ident);
   if(pWork)
   {
      pWork++;
      i=0;
      while(*pWork&&!isspace(*pWork))
      {
         bucket[i]=*pWork;
         pWork++;
         i++;
      }
      bucket[i]=0;
      p=0;
      *outval=0;
      for(i=strlen(bucket)-1;i>=0;i--,p+=4)
         *outval=*outval+((bucket[i]-'0')<<p);
   }
}

void disablestereo(void)
{
   outportb(sbbase+4,0x0e);
   outportb(sbbase+5,0);
}

void setsbvect(int intin)
{
   if(intin>7)
      sbvect=intin+104;
   else
      sbvect=intin+8;
   newVect.pm_offset=(int)_sbwrap;
   newVect.pm_selector=CodeSeg;
   _go32_dpmi_get_protected_mode_interrupt_vector(sbvect,&oldVect);
   _go32_dpmi_set_protected_mode_interrupt_vector(sbvect,&newVect);
}

