#include "main.h"
#include "globals.h"
#include "galvid.h"
#include "blit.h"
#include "scobrvid.h"
#include "scramvid.h"
#include "amdvid.h"
#include "twk.h"

extern int starsblink;

static int scobrainten;
static int rescue_int_en;

#define STARS_COLOR_BASE 		32
#define BULLETS_COLOR_BASE		(STARS_COLOR_BASE + 64)
#define BACKGROUND_COLOR_BASE   (BULLETS_COLOR_BASE + 2)

void initscobra(void)
{
   precalcstars();
   blnkchar=16;
   maxchar=32;
   startchar=0;
   pCharMem=&pCode[0x8800];
   pAttrMem=&pCode[0x9000];
   scobrainten=0;
}

void inithustler(void)
{
   int i;

   for(i=0;i<0x4000;i++)
	{
      BYTE xormask;
		int bits[8];
      int j;

      for(j=0;j<8;j++)
         bits[j]=(i>>j)&1;

      xormask=0xff;
      if(bits[0]^bits[1])
         xormask^=0x01;
      if(bits[3]^bits[6])
         xormask^=0x02;
      if(bits[4]^bits[5])
         xormask^=0x04;
      if(bits[0]^bits[2])
         xormask^=0x08;
      if(bits[2]^bits[3])
         xormask^=0x10;
      if(bits[1]^bits[5])
         xormask^=0x20;
      if(bits[0]^bits[7])
         xormask^=0x40;
      if(bits[4]^bits[6])
         xormask^=0x80;

      pBaseCode[0][i]^=xormask;
	}


   for(i=0;i<0x0800;i++)
      pBaseCode[1][i]=(pBaseCode[1][i]&0xfc)|((pBaseCode[1][i]&1)<<1)|((pBaseCode[1][i]&2)>>1);

   initscobra();

}

void scobraupdatescreen(void)
{
   fastclear();

   scramdispchars();
   drawsprites(0x9040,32);
   drawscrambullets(0x9060);
   if(pCode[0xa804])
      drawstars(FALSE,TRUE);
}

void calipsoupdatescreen(void)
{
   fastclear();
   scramdispchars();
   drawscrambullets(0x9060);
   drawnoflipsprites(0x9040,32);
}

void hustlerupdatescreen(void)
{
   amddispchars();
   drawamdsprites(0x9040);
}

void scobraint(void)
{
   static int count=0;

   if (count >= 45)
	{
      count=0;
      starsblink=(starsblink+1)&3;
	}
   count++;
   if(scobrainten)
      mz80nmi();
}


void scobravw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   scobrainten=value;
   if(!scobrainten)
      mz80ClearPendingInterrupt();
}

void initsuperbon(void)
{
	/*
	*   Code rom deryption worked out by hand by Chris Hardy.
	*/
	int i;

	for (i = 0;i < 0x1000;i++)
	{
		/* Code is encrypted depending on bit 7 and 9 of the address */
		switch (i & 0x280)
		{
			case 0x000:
            pCode[i] ^= 0x92;
				break;
			case 0x080:
            pCode[i] ^= 0x82;
				break;
			case 0x200:
            pCode[i] ^= 0x12;
				break;
			case 0x280:
            pCode[i] ^= 0x10;
				break;
		}
	}
   initscobra();
}


void anteaterdecodegfx(pBYTE pIn,int nType)
{
   int i,j;
   PRECALC_RETURN

	/*
	*   Code To Decode Lost Tomb by Mirko Buffoni
	*   Optimizations done by Fabio Buffoni
	*/



	/* The gfx ROMs are scrambled. Decode them. They have been loaded at 0x1000, */
	/* we write them at 0x0000. */

	for (i = 0;i < 0x1000;i++)
	{
		j = i & 0x9bf;
		j |= ( bit(i,4) ^ bit(i,9) ^ ( bit(i,2) & bit(i,10) ) ) << 6;
		j |= ( bit(i,2) ^ bit(i,10) ) << 9;
		j |= ( bit(i,0) ^ bit(i,6) ^ 1 ) << 10;
      pIn[i] = pIn[j + 0x1000];
	}
}

void losttombdecodegfx(pBYTE pIn,int nType)
{
	/*
	*   Code To Decode Lost Tomb by Mirko Buffoni
	*   Optimizations done by Fabio Buffoni
	*/
	int i,j;
   PRECALC_RETURN


	/* The gfx ROMs are scrambled. Decode them. They have been loaded at 0x1000, */
	/* we write them at 0x0000. */

	for (i = 0;i < 0x1000;i++)
	{
		j = i & 0xa7f;
		j |= ( (bit(i,1) & bit(i,8)) | ((1 ^ bit(i,1)) & (bit(i,10)))) << 7;
		j |= ( bit(i,7) ^ (bit(i,1) & ( bit(i,7) ^ bit(i,10) ))) << 8;
		j |= ( (bit(i,1) & bit(i,7)) | ((1 ^ bit(i,1)) & (bit(i,8)))) << 10;
      pIn[i] = pIn[j + 0x1000];
	}
}



static int bit(int i,int n)
{
	return ((i >> n) & 1);
}



void initrescue(void)
{
  //precalcstars();
   blnkchar=16;
   maxchar=32;
   startchar=0;
   pCharMem=&pCode[0x8800];
   pAttrMem=&pCode[0x9000];
   rescue_int_en=0;
   clearvesamem(34);
}



void setrescuepal(pBYTE pIn)
{
   int i,r,g,b,bits;
   int bit0,bit1,bit2;
   int map[4] = { 0x00, 0x88, 0xcc, 0xff };
   BYTE color;
   for(i=0;i< 32  ;i++)
   {
      outportb(0x3c8,i);

//red
      bit0=(*pIn&0x01)*0x21;
      bit1=((*pIn>>1)&0x01)*0x47;
      bit2=((*pIn>>2)&0x01)*0x97;
      outportb(0x3c9,(bit0+bit1+bit2)>>2);
//green
      bit0=((*pIn>>3)&0x01)*0x21;
      bit1=((*pIn>>4)&0x01)*0x47;
      bit2=((*pIn>>5)&0x01)*0x97;
      outportb(0x3c9,(bit0+bit1+bit2)>>2);
//blue
      bit1=((*pIn>>6)&0x01)*0x4f;
      bit2=((*pIn>>7)&0x01)*0xa8;
      outportb(0x3c9,(bit1+bit2)>>2);
      pIn++;

	}
	for (i = 0;i < 64;i++)
	{
		outportb(0x3c8,i + STARS_COLOR_BASE);

		bits = (i >> 0) & 0x03;
		r = map[bits];
		outportb(0x3c9,r);

		bits = (i >> 2) & 0x03;
		g = map[bits];
		outportb(0x3c9,g);

		bits = (i >> 4) & 0x03;
		b = map[bits];
		outportb(0x3c9,b);
	}

	for (i = 0; i <  128; i++)
	{
		int r = 0;
		int g = i;
		int b = i << 1 ;

        outportb(0x3c8,i+BACKGROUND_COLOR_BASE);

        outportb(0x3c9,r);
		outportb(0x3c9,g);
		outportb(0x3c9,b);

	}
  // clearvesamem(34);
}


void rescueupdatescreen(void)
{
	fastclear();
	scramdispchars();
	drawsprites(0x9040,32);
	drawscrambullets(0x9060);
   if(pCode[0xa804])
      drawstars(FALSE,TRUE);
}

void rescueint(void)
{
   if(rescue_int_en)
      mz80nmi(); 
}

void rescuevw(UINT32 offset, UINT8 value, struct MemoryWriteByte * mwb)
{
   rescue_int_en=value;
   if(!rescue_int_en)
      mz80ClearPendingInterrupt();
}


void rescuedecodegfx(pBYTE pIn,int nType)
{
   int i,j;
   PRECALC_RETURN


	/* The gfx ROMs are scrambled. Decode them. They have been loaded at 0x1000, */
	/* we write them at 0x0000. */
	memset(pIn,0,0x1000);
	for (i = 0;i < 0x1000;i++)
	{	
		j = i & 0xa7f;

		j |= ( bit(i,3) ^ bit(i,10) ) << 7;
		j |= ( bit(i,1) ^ bit(i,7) ) << 8;
		j |= ( bit(i,0) ^ bit(i,8) ) << 10;

	    pIn[i] = pIn[j + 0x1000];
	}
}

